%   
% G1 - ᒺ, NTask - 
% NMethod=1 - Ziegler-Nichols
% NMethod=2 - Cohen-Coon
% NMethod=3 - Chien-Hoones-Reswick
% NMethod=4 - pidtool
%bPIDType variable for different PID structures
%   1 for P, 2 for PI, 3 for normal PID
%   4 for PID with D in feedback
%   5 for refined Ziegler-Nichols tuning structure
%   6,7,8  
function [Gc_Sys,Kp,Ti,Td,Kd] = design(G1,bPIDType,NMethod)
[k,L,T]=get_fod(G1,1); %first-order plus delay approximation
T0=T
NN=10; %    
bOvshoot=0; % 
Kd=0;
switch NMethod
   case 1 
       [Gc_Sys,H_Sys,Kp,Ti,Td]=ziegler_nic(bPIDType,[k,L,T,NN]);
   case 2       
       [Gc_Sys,H_Sys,Kp,Ti,Td]=cohen_pid(bPIDType,[k,L,T,NN]);
   case 3       
       [Gc_Sys,H_Sys,Kp,Ti,Td]=chr_pid(bPIDType,1,[k,L,T,NN,bOvshoot]);
   case 0       
    switch bPIDType
      case 6 %   PID-D
       [Gc_Sys,Kp,Ti,Td,Kd]=blonsky1(G1,k,T);
      case 7 %   PID-2D
       [Gc_Sys,Kp,Ti,Td,Kd]=blonsky2(G1,k,T);
      case 8 %   PID-3D
       [Gc_Sys,Kp,Ti,Td,Kd]=blonsky3(G1,k,T);
    end
end

       
%--------------------------------------------------------------

%----------------------------------------------------------------------------------
% get_fod    ᒺ,   1-
%     
%  -    ᒺ
% L - 
% 
%----------------------------------------------------------------------------------
function [K,L,T]=get_fod(G1,method)

K=real(dcgain(G1)); 
if nargin==1 %       get_fod
   %      ()   Pm
   % wc,wcp -   (  0    180 ) 
   [Kc,Pm,wc,wcp]=margin(G1); 
   ikey=0; 
   L=1.6*pi/(3*wc); T=0.5*Kc*K*L; 
   if isfinite(Kc), %    
       x0=[L;T]; 
       while ikey==0, 
          ww1=wc*x0(1); ww2=wc*x0(2);
          FF=[K*Kc*(cos(ww1)-ww2*sin(ww1))+1+ww2^2; sin(ww1)+ww2*cos(ww1)];
          J=[-K*Kc*wc*sin(ww1)-K*Kc*wc*ww2*cos(ww1),-K*Kc*wc*sin(ww1)+2*wc*ww2;
              wc*cos(ww1)-wc*ww2*sin(ww1), wc*cos(ww1)]; 
          x1=x0-inv(J)*FF; 
          if norm(x1-x0)<1e-8, ikey=1; else, x0=x1; end
       end
       L=x0(1); T=x0(2);
   end
elseif nargin==2 & method==1 %   
   [nn,dd]=tfdata(G1,'v'); [n1,d1]=tf_derv(nn,dd); [n2,d2]=tf_derv(n1,d1);
   K1=n1(end)/d1(end); K2=n2(end)/d2(end); Tar=-K1/K; T=sqrt(K2/K-Tar^2); L=Tar-T;
end
L=real(L); T=real(T);
%------------------------------------------------------------------------------

%----------------------------------------------------------------------------
%tf_derv      
%----------------------------------------------------------------------------
function [e,f]=tf_derv(b,a)
f=conv(a,a);
e1=conv((length(b)-1:-1:1).*b(1:end-1),a); e2=conv((length(a)-1:-1:1).*a(1:end-1),b);
L0=length(e1)-length(e2); e=[zeros(1,-L0) e1]-[zeros(1,L0) e2];

%------------------------------------------------------------------------------
%blonsky2 -   ϲ-
%------------------------------------------------------------------------------
function [Gc_Sys,Kp,Ti,Td,Kd]=blonsky2(Wo,K0,T0)
% Wo.userdata -    ᒺ T1<T2
if Wo.userdata(1)<Wo.userdata(2)
   T1=Wo.userdata(1);T2=Wo.userdata(2);
else
   T1=Wo.userdata(1);T2=Wo.userdata(2);
end
tau=Wo.inputd % 
Td=(0.4/K0)*tau;
if tau/T0<=0.3
    Kp1=(0.298-72.0967*(tau/T0))/(1.0-18.0885*(tau/T0));
    Ti1=(0.598+1.477*(tau/T0)+12.857*(tau/T0)*(tau/T0))*(T0/18.5);
    Td1=(26.2-72.4857*(tau/T0)+105.714*(tau/T0)*(tau/T0))*(T0/18.5);
    Kd1=((1.6443-16.636*(tau/T0))/(1-12.6592*(tau/T0)))*(T0/15);
end
if tau/T0>0.3
    Kp1=(9.9878-21.2*(tau/T0)+12.982*(tau/T0)*(tau/T0));
    Ti1=(0.583-1.12*(tau/T0)+20.5714*(tau/T0)*(tau/T0))*(T0/18.5);
    Td1=(15.267-1.5929*(tau/T0)-8.2143*(tau/T0)*(tau/T0))*(T0/18.5);
    Kd1=(3.4798-6.2025*(tau/T0)+2.9296*(tau/T0)*(tau/T0))*(T0/18.5);
end
Kp=Kp1/K0;
if T1/T2<=0.4
    Ti=Ti1*(0.764+1.6298*(T1/T2)-2.5973*(T1/T2)*(T1/T2))*K0;
    Td=Td1*(0.7962-0.1333*(T1/T2)+0.9416*(T1/T2)*(T1/T2))/K0;
    Kd=Kd1*(0.8304+0.4262*(T1/T2));
end
if T1/T2>0.4
    Ti=Ti1*(1.4235-1.7065*(T1/T2)+1.615*(T1/T2)*(T1/T2))*K0;
    Td=Td1*(1.2068-0.8698*(T1/T2)+0.877*(T1/T2)*(T1/T2))/K0;
    Kd=Kd1*(0.8333+0.4167*(T1/T2));
end
Gc_Sys=tf([Td*Kd*Ti, Td*Ti, (Kp*+Kd), 1],[Ti, 0]);


%------------------------------------------------------------------------------
%blonsky3 -    --3
%------------------------------------------------------------------------------
function [Gc_Sys,Kp,Ti,Td,Kd]=blonsky3(Wo,K0,T0)
% Wo.userdata -    ᒺ T1<T2
if Wo.userdata(1)<Wo.userdata(2)
   T1=Wo.userdata(1);T2=Wo.userdata(2);
else
   T1=Wo.userdata(1);T2=Wo.userdata(2);
end
tau=Wo.inputd % 
Td=(0.4/K0)*tau;
if tau/T0<=0.3
    Kp1=(1.1857/(tau/T0))-0.3568;
    Ti1=(-0.026+3.42*(tau/T0)+14*(tau/T0)*(tau/T0))*(T0/18.64);
    Td1=(15.22+1.5286*(tau/T0)-38.57*(tau/T0)*(tau/T0))*(T0/18.64);
    Kd1=(-0.385+3.8657*(tau/T0)+0.2857*(tau/T0)*(tau/T0))/(T0/18.64);
end
if tau/T0>0.3
    Kp1=(10.6709-5.3255*(tau/T0))/(1.0+4.8737*(tau/T0));
    Ti1=(-0.7359+8.6177*(tau/T0))/(1-0.6453*(tau/T0))*(T0/18.64);
    Td1=(12.7315-14.0461*(tau/T0))/(1-1.0065*(tau/T0))*(T0/18.64);
    Kd1=(0.32+1.9*(tau/T0)-(tau/T0)*(tau/T0))/(T0/18.64);
end
if T1/T2<=0.4
    Ti=Ti1*(0.8895+0.3494*(T1/T2)-0.1857*(T1/T2)*(T1/T2))*K0;
end
if T1/T2>0.4
    Ti=Ti1*(0.7403+0.8335*(T1/T2)-0.4625*(T1/T2)*(T1/T2))*K0;
end
if K0>0.3
    Kp=Kp1/K0;
    Td=Td1/K0;
    Kd=Kd1;
end
Gc_Sys=tf([Td*Kd*Ti, Td*Ti, (Kp*+Kd), 1],[Ti, 0]);


%------------------------------------------------------------------------------
%blonsky1 -     
%------------------------------------------------------------------------------
function [Gc_Sys,Kp,Ti,Td,Kd]=blonsky1(Wo,K0,T0)
% Wo.userdata -    ᒺ T1<T2
if Wo.userdata(1)<Wo.userdata(2)
   T1=Wo.userdata(1);T2=Wo.userdata(2);
else
   T1=Wo.userdata(1);T2=Wo.userdata(2);
end
tau=Wo.inputd % 
Td=(0.4/K0)*tau;
if tau/T0<=0.3
    Kp1=(-7.1742-26.9352*(tau/T0))/(1.0-22.2649*(tau/T0));
    Ti1=((0.5622+2.2248*(tau/T0))/(1.0-2.2671*(tau/T0)))*(T0/18.64);
    Kd1=(3.792+9.3486*(tau/T0)-52.5714*(tau/T0)*(tau/T0))*(T0/18.64);
end
if tau/T0>0.3
    Kp1=(-2.5682-4.3441*(tau/T0))/(1.0-8.1132*(tau/T0));
    Ti1=(-5.008+32.0995*(tau/T0)-8.814*(tau/T0)*(tau/T0))*(T0/18.64);
    Kd1=(3.4798-6.2025*(tau/T0)+2.9296*(tau/T0)*(tau/T0))*(T0/18.64);
end
if T1/T2<=0.4
    Kp=Kp1*(1.497-1.26*(T1/T2))/K0;
    Ti=Ti1*(0.4305+1.4326*(T1/T2))*K0;
    Kd=Kd1;
end
if T1/T2>0.4
    Kp=Kp1/K0;
    Ti=Ti1*(0.91+0.2277*(T1/T2))*K0;
    Kd=Kd1;
end
Gc_Sys=tf([Td*Kd, Td, Kp+Kd/Ti],[0, 1]);




%------------------------------------------------------------------------------
%ziegler_nic is used to design PID controllers using the Ziegler-Nichols algorithm.
%------------------------------------------------------------------------------
function [Gc_Sys,H_Sys,Kp,Ti,Td]=ziegler_nic(key,vars)
Ti=0; Td=0;
switch length(vars)
case 3, %     3 
   K=vars(1); Tc=vars(2); N=vars(3);
   switch key
   case 1, Kp=0.5*K; 
   case 2, Kp=0.4*K; Ti=0.8*Tc; 
   case {3,4}, Kp=0.6*K; Ti=0.5*Tc; Td=0.12*Tc; 
   end
case 4,  %     4 
   K=vars(1); L=vars(2); T=vars(3); N=vars(4); a=K*L/T; 
   switch key
   case 1, key==1,  Kp=1/a; % P-
   case 2, Kp=0.9/a; Ti=3.33*L; % PI-
   case {3,4}, Kp=1.2/a; Ti=2*L; Td=L/2; % PID-  PIDF-
   end
case 5, 
   K=vars(1); Tc=vars(2); rb=vars(3); pb=pi*vars(4)/180; 
   N=vars(5); Kp=K*rb*cos(pb); 
   if key==2, Ti=-Tc/(2*pi*tan(pb)); 
   elseif key==3 | key==4, Ti=Tc*(1+sin(pb))/(pi*cos(pb)); Td=Ti/4; end
end
switch key
case 1, Gc_Sys=tf(Kp,1); H_Sys=[]; % P-
case 2, Gc_Sys=tf(Kp*[Ti,1],[Ti,0]); H_Sys=[]; % P-
case 3 % PID-
   dd=Ti*[Td/N,1,0]; nn=[Kp*Ti*Td*(N+1)/N, Kp*(Ti+Td/N), Kp];
   Gc_Sys=tf(nn,dd); %  
   H_Sys=[];
case 4 % PDF-
   d0=sqrt(Ti*(Ti-4*Td)); Ti0=Ti; 
   Kp=0.5*(Ti+d0)*Kp/Ti; Ti=0.5*(Ti+d0); Td=Ti0-Ti; nn=Kp*[Ti,1]; dd=[Ti,0]; 
   nH=[(1+Kp/N)*Ti*Td, Kp*(Ti+Td/N), Kp]; dH=Kp*conv([Ti,1],[Td/N,1]);
   Gc_Sys=tf(nn,dd); H_Sys=tf(nH,dH);
end

%------------------------------------------------------------------------
%cohen_pid is use to design PID controller using the Cohen-Coon algorithm.
%------------------------------------------------------------------------
function [Gc_Sys,H_Sys,Kp,Ti,Td]=cohen_pid(key,vars)
Ti=0; Td=0; K=vars(1); L=vars(2); T=vars(3); N=vars(4); a=K*L/T; tau=L/(L+T); 
switch key
case 1, Gc_Sys=tf((1+0.35*tau/(1-tau))/a,1); H_Sys=[];
case 2, 
   Kp=0.9*(1+0.92*tau/(1-tau))/a; Ti=(3.3-3*tau)*L/(1+1.2*tau);
   Gc_Sys=tf(Kp*[Ti,1],[Ti,0]); H_Sys=[];
case {3,4}
   Kp=1.35*(1+0.18*tau/(1-tau))/a; Ti=(2.5-2*tau)*L/(1-0.39*tau); Td=0.37*(1-tau)*L/(1-0.81*tau); 
   if key==3
      dd=Ti*[Td/N,1,0]; nn=[Kp*Ti*Td*(N+1)/N, Kp*(Ti+Td/N), Kp]; Gc_Sys=tf(nn,dd); H_Sys=[];
   elseif key==4
      d0=sqrt(Ti*(Ti-4*Td)); Ti0=Ti; Kp=0.5*(Ti+d0)*Kp/Ti; Ti=0.5*(Ti+d0); Td=Ti0-Ti;
      Gc_Sys=tf(Kp*[Ti,1],[Ti,0]); nH=[(1+Kp/N)*Ti*Td, Kp*(Ti+Td/N), Kp]; 
      dH=Kp*conv([Ti,1],[Td/N,1]); H_Sys=tf(nH,dH);
   end
case 5
   Kp=1.24*(1+0.13*tau/(1-tau))/a; Td=(0.27-0.36*tau)*L/(1-0.87*tau); 
   dd=[Td/N,1]; nn=Kp*[Td*(N+1)/N, 1]; Gc_Sys=tf(nn,dd); H_Sys=[];
end

%------------------------------------------------------------------------------
%chr_pid is used to design PID using the Chien-Hoones-Reswick tuning algorithm.
%------------------------------------------------------------------------------
function [Gc_Sys,H_Sys,Kp,Ti,Td]=chr_pid(key,typ,vars)
Ti=0; Td=0; K=vars(1); L=vars(2); T=vars(3); N=vars(4); 
a=K*L/T; ovshoot=vars(5);
if typ==1, TT=T; else TT=L; typ=2;  end
if ovshoot==0, 
   KK=[0.3,0.35,1.2,0.6,1,0.5; 0.3,0.6,4,0.95,2.4,0.42];
else, 
   KK=[0.7,0.6,1,0.95,1.4,0.47; 0.7,0.7,2.3,1.2,2,0.42]; 
end
switch key 
case 1, Kp=KK(typ,1)/a; Gc_Sys=tf(Kp,1); H_Sys=[];
case 2, Kp=KK(typ,2)/a; Ti=KK(typ,3)*TT; Gc_Sys=tf(Kp*[Ti,1],[Ti,0]); H_Sys=[]; 
otherwise
   Kp=KK(typ,4)/a; Ti=KK(typ,5)*TT; Td=KK(typ,6)*L; 
   if key==3
      dd=Ti*[Td/N,1,0]; nn=[Kp*Ti*Td*(N+1)/N, Kp*(Ti+Td/N), Kp]; Gc_Sys=tf(nn,dd); H_Sys=[];
   elseif key==4
      d0=sqrt(Ti*(Ti-4*Td)); Ti0=Ti; Kp=0.5*(Ti+d0)*Kp/Ti; Ti=0.5*(Ti+d0); Td=Ti0-Ti;
      nH=[(1+Kp/N)*Ti*Td, Kp*(Ti+Td/N), Kp]; dH=Kp*conv([Ti,1],[Td/N,1]);
      Gc_Sys=tf(Kp*[Ti,1],[Ti,0]); H_Sys=tf(nH,dH);
   end
end

